﻿using System;
using System.Collections.ObjectModel;
using Unclassified.UI;

namespace ViewCommandExample.ViewModel
{
	class MainViewModel : ViewModelBase, IViewCommandSource
	{
		#region Private data

		private int counter;

		#endregion Private data

		#region Constructors

		/// <summary>
		/// Initialises a new instance of the MainViewModel class.
		/// </summary>
		public MainViewModel()
		{
			InitializeCommands();

			// Focus the Add button right at the start of the application.
			// Since the MainWindow view does surely not have its DataContext assigned to this
			// instance yet (we're still in the constructor) and thus the view has not registered
			// in this ViewModel yet, the view command invokation must be delayed until all loading
			// has finished.
			ViewCommandManager.InvokeLoaded("FocusAdd");
		}

		#endregion Constructors

		#region Commands

		public DelegateCommand AddCommand { get; private set; }
		public DelegateCommand AddFocusCommand { get; private set; }
		public DelegateCommand RemoveCommand { get; private set; }

		private void InitializeCommands()
		{
			AddCommand = new DelegateCommand(OnAdd);
			AddFocusCommand = new DelegateCommand(OnAddFocus);
			RemoveCommand = new DelegateCommand(OnRemove, CanRemove);
		}

		private void OnAdd()
		{
			// Add a new item to the list.
			Items.Add((counter++).ToString() + "  " + DateTime.Now.ToString());
			// The Remove command may now be available, request rechecking.
			RemoveCommand.RaiseCanExecuteChanged();
		}

		private void OnAddFocus()
		{
			// Add a new item to the list.
			Items.Add((counter++).ToString() + "  " + DateTime.Now.ToString());
			// The Remove command may now be available, request rechecking.
			RemoveCommand.RaiseCanExecuteChanged();
			// Tell the view to focus the list.
			// Since everything is set up by now, the command can be invoked
			// synchronously.
			ViewCommandManager.Invoke("FocusList");
		}

		private bool CanRemove()
		{
			// Only allow removing items if there are any.
			return Items.Count > 0;
		}

		private void OnRemove()
		{
			if (Items.Count > 0)
			{
				// Remove the last item from the list.
				Items.RemoveAt(Items.Count - 1);
				// The Remove command may no longer be available, request rechecking.
				RemoveCommand.RaiseCanExecuteChanged();
			}
		}

		#endregion Commands

		#region Data properties

		private ObservableCollection<string> items = new ObservableCollection<string>();
		public ObservableCollection<string> Items { get { return items; } }

		#endregion Data properties

		#region IViewCommandSource

		private ViewCommandManager viewCommandManager = new ViewCommandManager();

		/// <summary>
		/// Gets the ViewCommandManager instance.
		/// </summary>
		public ViewCommandManager ViewCommandManager { get { return viewCommandManager; } }

		#endregion IViewCommandSource
	}
}
